@@ -28,12 +28,13 @@ try:
     from .Backend import Backend
     from .BackendPandaPower import PandaPowerBackend
     from .Parameters import Parameters
-    from .ChronicsHandler import ChronicsHandler, Multifolder, GridStateFromFileWithForecasts, GridValue
+    from .ChronicsHandler import ChronicsHandler, Multifolder, GridStateFromFileWithForecasts, GridValue, ChangeNothing
     from .Action import Action, TopologyAction, TopoAndRedispAction
     from .Exceptions import *
     from .Observation import CompleteObservation, Observation
     from .Reward import FlatReward, Reward, L2RPNReward, RedispReward
     from .GameRules import LegalAction, AllwaysLegal, DefaultRules
+    from .VoltageControler import ControlVoltageFromFile
 
     from .Settings_L2RPN2019 import L2RPN2019_CASEFILE, L2RPN2019_DICT_NAMES, ReadPypowNetData, CASE_14_L2RPN2019_LAYOUT
     from .Settings_5busExample import EXAMPLE_CHRONICSPATH, EXAMPLE_CASEFILE, CASE_5_GRAPH_LAYOUT
@@ -46,7 +47,7 @@ except (ModuleNotFoundError, ImportError):
     from Backend import Backend
     from BackendPandaPower import PandaPowerBackend
     from Parameters import Parameters
-    from ChronicsHandler import ChronicsHandler, Multifolder, GridStateFromFileWithForecasts, GridValue
+    from ChronicsHandler import ChronicsHandler, Multifolder, GridStateFromFileWithForecasts, GridValue, ChangeNothing
     from Action import Action, TopologyAction, TopoAndRedispAction
     from Exceptions import *
     from Observation import CompleteObservation, Observation
@@ -57,6 +58,7 @@ except (ModuleNotFoundError, ImportError):
     from Settings_case14_redisp import case14_redisp_CASEFILE, case14_redisp_CHRONICSPATH, case14_redisp_TH_LIM
     from Settings_case14_test import case14_test_CASEFILE, case14_test_CHRONICSPATH, case14_test_TH_LIM
     from Settings_case14_realistic import case14_real_CASEFILE, case14_real_CHRONICSPATH, case14_real_TH_LIM
+    from VoltageControler import ControlVoltageFromFile
 
 import pdb
 
@@ -176,8 +178,7 @@ def _get_default_aux(name, kwargs, defaultClassApp, _sentinel=None,
                 try:
                     res = defaultClass(**build_kwargs)
                 except Exception as e:
-                    err_msg = "Cannot create and instance of {} with parameters \"{}\""
-                    print(err_msg.format(name, defaultClass, build_kwargs))
+                    e.args = e.args + ("Cannot create and instance of {} with parameters \"{}\"".format(defaultClass, build_kwargs),)
                     raise
             elif defaultinstance is not None:
                 if len(build_kwargs):
@@ -250,9 +251,15 @@ def make(name_env="case14_realistic", **kwargs):
         The type of chronics that represents the dynamics of the Environment created. Usually they come from different
         folders.
 
+    data_feeding: ``type``, optional
+        The type of chronics handler you want to use.
+
     chronics_path: ``str``
         Path where to look for the chronics dataset.
 
+    volagecontroler_class: ``type``, optional
+        The type of :class:`grid2op.VoltageControler.VoltageControler` to use, it defaults to
+
     Returns
     -------
     env: :class:`grid2op.Environment.Environment`
@@ -293,9 +300,9 @@ def make(name_env="case14_realistic", **kwargs):
                                      msg_error=msg_error)
 
     # bulid the default parameters for each case file
-    defaultinstance_chronics_kwargs = {}
     data_feeding_default_class = ChronicsHandler
     gamerules_class = AllwaysLegal
+    defaultinstance_chronics_kwargs = {}
     if name_env.lower() == "case14_fromfile":
         default_grid_path = CASE_14_FILE
         if chronics_path == '':
@@ -375,11 +382,20 @@ def make(name_env="case14_realistic", **kwargs):
         default_action_class = TopoAndRedispAction
         default_reward_class = RedispReward
         gamerules_class = DefaultRules
+    elif name_env.lower() == "blank":
+        default_name_converter = {}
+        default_grid_path = ""
+        default_action_class = TopologyAction
+        default_reward_class = L2RPNReward
+        gamerules_class = AllwaysLegal
     else:
         raise UnknownEnv("Unknown Environment named \"{}\". Current known environments are \"case14_fromfile\" "
                          "(default), \"case5_example\", \"case14_redisp\", \"case14_realistic\" "
                          "and \"l2rpn_2019\"".format(name_env))
 
+    if "chronicsClass" not in defaultinstance_chronics_kwargs:
+        defaultinstance_chronics_kwargs["chronicsClass"] = ChangeNothing
+
     # extract powergrid dependant parameters
     ## type of rules of the game (mimic the operationnal constraints)
     msg_error = "The type of rules of the environment (keyword \"gamerules_class\")"
@@ -421,19 +437,41 @@ def make(name_env="case14_realistic", **kwargs):
     msg_error = "The argument to build the data generation process [chronics] (keyword \"data_feeding_kwargs\")"
     msg_error += " should be a dictionnary."
     data_feeding_kwargs = _get_default_aux("data_feeding_kwargs", kwargs,
-                                 defaultClassApp=dict, defaultinstance=defaultinstance_chronics_kwargs,
-                                 msg_error=msg_error)
-
+                                           defaultClassApp=dict,
+                                           defaultinstance=defaultinstance_chronics_kwargs,
+                                           msg_error=msg_error)
+    for el in defaultinstance_chronics_kwargs:
+        if not el in data_feeding_kwargs:
+            data_feeding_kwargs[el] = defaultinstance_chronics_kwargs[el]
 
     ### the chronics generator
     msg_error = "The argument to build the data generation process [chronics] (keyword \"chronics_class\")"
+    msg_error += " should be a class that inherit grid2op.ChronicsHandler.GridValue."
+    chronics_class_used = _get_default_aux("chronics_class", kwargs,
+                                           defaultClassApp=GridValue,
+                                           defaultClass=data_feeding_kwargs["chronicsClass"],
+                                           msg_error=msg_error,
+                                           isclass=True)
+    data_feeding_kwargs["chronicsClass"] = chronics_class_used
+
+    ### the chronics generator
+    msg_error = "The argument to build the data generation process [chronics] (keyword \"data_feeding\")"
     msg_error += " should be a class that inherit grid2op.ChronicsHandler.ChronicsHandler."
-    data_feeding = _get_default_aux("chronics_class", kwargs,
+    data_feeding = _get_default_aux("data_feeding", kwargs,
                                     defaultClassApp=ChronicsHandler,
                                     defaultClass=data_feeding_default_class,
                                     build_kwargs=data_feeding_kwargs,
                                     msg_error=msg_error)
 
+    ### controler for voltages
+    msg_error = "The argument to build the online controler for chronics (keyword \"volagecontroler_class\")"
+    msg_error += " should be a class that inherit grid2op.VoltageControler.ControlVoltageFromFile."
+    volagecontroler_class = _get_default_aux("volagecontroler_class", kwargs,
+                                             defaultClassApp=ControlVoltageFromFile,
+                                             defaultClass=ControlVoltageFromFile,
+                                             msg_error=msg_error,
+                                             isclass=True)
+
     if not os.path.exists(grid_path):
         raise EnvError("There is noting at \"{}\" where the powergrid should be located".format(
             os.path.abspath(grid_path)))
@@ -446,7 +484,8 @@ def make(name_env="case14_realistic", **kwargs):
                       actionClass=action_class,
                       observationClass=observation_class,
                       rewardClass=reward_class,
-                      legalActClass=gamerules_class
+                      legalActClass=gamerules_class,
+                      voltagecontrolerClass=volagecontroler_class
                       )
 
     # update the thermal limit if any
@@ -461,4 +500,4 @@ def make(name_env="case14_realistic", **kwargs):
         env.graph_layout = CASE_14_L2RPN2019_LAYOUT
     elif name_env.lower() == "case5_example":
         env.graph_layout = CASE_5_GRAPH_LAYOUT
-    return env
\ No newline at end of file
+    return env
